#!/usr/bin/env python3

import socketserver
import signal
import time
import os
from base64 import b64decode
from base64 import b64encode

from secrets import FLAG

IP_ADDR = "0.0.0.0"
PORT = 4321

# ------------------------------------------------------

def xor(plain_bytes, key_bytes):
    cipher_bytes = [0] * len(plain_bytes)
    for i in range(0, len(plain_bytes)):
        cipher_bytes[i] = plain_bytes[i] ^ key_bytes[i % len(key_bytes)]
    return bytes(cipher_bytes)

def wrap(data):
    return b64encode(data)

def unwrap(data):
    return b64decode(data)

# ------------------------------------------------------
    
def challenge(req):
    global FLAG
    
    random_bytes = os.urandom(len(FLAG))
    encrypted_flag = xor(FLAG, random_bytes)
    
    req.sendall(b'Welcome to Many-times pad.\n\n')
    time.sleep(1)
    
    while True:    
        try:
            req.sendall(b'Make a choice:\n' +\
                        b' [1] Show flag\n' +\
                        b' [2] Encrypt\n\n')

            opt = req.recv(4096).decode().strip()
            if opt == "1":
                data_out = b'Flag: ' + wrap(encrypted_flag) + b'\n'
                req.sendall(data_out)
            elif opt == "2":
                req.sendall(b'Send data to encrypt:\n')
                data_in = req.recv(4096).decode().strip()
                
                try:
                    plain_data = unwrap(data_in)
                except Exception as e:
                    req.sendall(b'[ERROR]: Could not understand data: make sure to base64 your payload!\n')
                    continue
                    
                encrypted_data = xor(plain_data, random_bytes)
                data_out = b'Encrypted: ' + wrap(encrypted_data) + b'\n'
                req.sendall(data_out)
            else:
                req.sendall(b'[ERROR]: Baaad choice\n')

        except Exception as e:
            exit(1)
        
class incoming(socketserver.BaseRequestHandler):
    def handle(self):
        signal.alarm(300)
        while True:
            challenge(self.request)

class ReusableTCPServer(socketserver.ForkingMixIn, socketserver.TCPServer):
    pass

# ------------------------------------------------------

def main():
    global IP_ADDR, PORT
    socketserver.TCPServer.allow_reuse_address = True
    server = ReusableTCPServer((IP_ADDR, PORT), incoming)
    print(f"Listening on ({IP_ADDR}, {PORT})")
    server.serve_forever()

if __name__ == "__main__":
    main()
